@@ -3,7 +3,7 @@ from __future__ import annotations
 from typing import TYPE_CHECKING, Any, NamedTuple
 
 from ..actions import ActionsPageResult, BoundAction, ResourceActionsClient
-from ..core import BoundModelBase, ClientEntityBase, Meta
+from ..core import BoundModelBase, Meta, ResourceClientBase
 from .domain import (
     Certificate,
     CreateManagedCertificateResponse,
@@ -15,7 +15,7 @@ if TYPE_CHECKING:
     from .._client import Client
 
 
-class BoundCertificate(BoundModelBase):
+class BoundCertificate(BoundModelBase, Certificate):
     _client: CertificatesClient
 
     model = Certificate
@@ -53,7 +53,13 @@ class BoundCertificate(BoundModelBase):
                Specifies how many results are returned by page
         :return: (List[:class:`BoundAction <hcloud.actions.client.BoundAction>`], :class:`Meta <hcloud.core.domain.Meta>`)
         """
-        return self._client.get_actions_list(self, status, sort, page, per_page)
+        return self._client.get_actions_list(
+            self,
+            status=status,
+            sort=sort,
+            page=page,
+            per_page=per_page,
+        )
 
     def get_actions(
         self,
@@ -68,7 +74,11 @@ class BoundCertificate(BoundModelBase):
                Specify how the results are sorted. Choices: `id` `id:asc` `id:desc` `command` `command:asc` `command:desc` `status` `status:asc` `status:desc` `progress` `progress:asc` `progress:desc` `started` `started:asc` `started:desc` `finished` `finished:asc` `finished:desc`
         :return: List[:class:`BoundAction <hcloud.actions.client.BoundAction>`]
         """
-        return self._client.get_actions(self, status, sort)
+        return self._client.get_actions(
+            self,
+            status=status,
+            sort=sort,
+        )
 
     def update(
         self,
@@ -83,7 +93,11 @@ class BoundCertificate(BoundModelBase):
                User-defined labels (key-value pairs)
         :return: :class:`BoundCertificate <hcloud.certificates.client.BoundCertificate>`
         """
-        return self._client.update(self, name, labels)
+        return self._client.update(
+            self,
+            name=name,
+            labels=labels,
+        )
 
     def delete(self) -> bool:
         """Deletes a certificate.
@@ -100,11 +114,11 @@ class BoundCertificate(BoundModelBase):
 
 class CertificatesPageResult(NamedTuple):
     certificates: list[BoundCertificate]
-    meta: Meta | None
+    meta: Meta
 
 
-class CertificatesClient(ClientEntityBase):
-    _client: Client
+class CertificatesClient(ResourceClientBase):
+    _base_url = "/certificates"
 
     actions: ResourceActionsClient
     """Certificates scoped actions client
@@ -114,7 +128,7 @@ class CertificatesClient(ClientEntityBase):
 
     def __init__(self, client: Client):
         super().__init__(client)
-        self.actions = ResourceActionsClient(client, "/certificates")
+        self.actions = ResourceActionsClient(client, self._base_url)
 
     def get_by_id(self, id: int) -> BoundCertificate:
         """Get a specific certificate by its ID.
@@ -122,7 +136,7 @@ class CertificatesClient(ClientEntityBase):
         :param id: int
         :return: :class:`BoundCertificate <hcloud.certificates.client.BoundCertificate>`
         """
-        response = self._client.request(url=f"/certificates/{id}", method="GET")
+        response = self._client.request(url=f"{self._base_url}/{id}", method="GET")
         return BoundCertificate(self, response["certificate"])
 
     def get_list(
@@ -157,9 +171,7 @@ class CertificatesClient(ClientEntityBase):
         if per_page is not None:
             params["per_page"] = per_page
 
-        response = self._client.request(
-            url="/certificates", method="GET", params=params
-        )
+        response = self._client.request(url=self._base_url, method="GET", params=params)
 
         certificates = [
             BoundCertificate(self, certificate_data)
@@ -190,7 +202,7 @@ class CertificatesClient(ClientEntityBase):
                Used to get certificate by name.
         :return: :class:`BoundCertificate <hcloud.certificates.client.BoundCertificate>`
         """
-        return self._get_first_by(name=name)
+        return self._get_first_by(self.get_list, name=name)
 
     def create(
         self,
@@ -219,7 +231,7 @@ class CertificatesClient(ClientEntityBase):
         }
         if labels is not None:
             data["labels"] = labels
-        response = self._client.request(url="/certificates", method="POST", json=data)
+        response = self._client.request(url=self._base_url, method="POST", json=data)
         return BoundCertificate(self, response["certificate"])
 
     def create_managed(
@@ -245,10 +257,10 @@ class CertificatesClient(ClientEntityBase):
         }
         if labels is not None:
             data["labels"] = labels
-        response = self._client.request(url="/certificates", method="POST", json=data)
+        response = self._client.request(url=self._base_url, method="POST", json=data)
         return CreateManagedCertificateResponse(
             certificate=BoundCertificate(self, response["certificate"]),
-            action=BoundAction(self._client.actions, response["action"]),
+            action=BoundAction(self._parent.actions, response["action"]),
         )
 
     def update(
@@ -272,7 +284,7 @@ class CertificatesClient(ClientEntityBase):
         if labels is not None:
             data["labels"] = labels
         response = self._client.request(
-            url=f"/certificates/{certificate.id}",
+            url=f"{self._base_url}/{certificate.id}",
             method="PUT",
             json=data,
         )
@@ -285,7 +297,7 @@ class CertificatesClient(ClientEntityBase):
         :return: True
         """
         self._client.request(
-            url=f"/certificates/{certificate.id}",
+            url=f"{self._base_url}/{certificate.id}",
             method="DELETE",
         )
         # Return always true, because the API does not return an action for it. When an error occurs a HcloudAPIException will be raised
@@ -323,12 +335,12 @@ class CertificatesClient(ClientEntityBase):
             params["per_page"] = per_page
 
         response = self._client.request(
-            url=f"/certificates/{certificate.id}/actions",
+            url=f"{self._base_url}/{certificate.id}/actions",
             method="GET",
             params=params,
         )
         actions = [
-            BoundAction(self._client.actions, action_data)
+            BoundAction(self._parent.actions, action_data)
             for action_data in response["actions"]
         ]
         return ActionsPageResult(actions, Meta.parse_meta(response))
@@ -365,7 +377,7 @@ class CertificatesClient(ClientEntityBase):
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url=f"/certificates/{certificate.id}/actions/retry",
+            url=f"{self._base_url}/{certificate.id}/actions/retry",
             method="POST",
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
